﻿Imports System
Imports System.Collections.Generic
Imports System.Linq
Imports System.Text
Imports System.Threading
Imports System.Threading.Tasks
Imports System.Windows
Imports System.Windows.Interop
Imports System.Windows.Threading

Namespace SolSoft.ObjectOrientedGuiStartup.WindowsForms
	''' <summary>
	''' HIDDEN BONUS!
	''' The "host" could also be turned into a class to host arbitrary "programs"
	''' </summary>
	Friend NotInheritable Class FinalMixedDesign
		''' <summary>
		''' The main entry point for the application.
		''' </summary>
		Private Sub New()
		End Sub
		<STAThread>
		Shared Function Main() As Integer
			'the host environment itself can be made into a class!
			Dim host As New WpfHostEnvironment()
			AddHandler host.UnhandledException, Sub(sender, e)
				System.Windows.MessageBox.Show(e.ExceptionObject.ToString())
				If e.IsTerminating Then
					host.ShutDown()
				End If
			End Sub

			'but it's hard; for example, if you move this constructor before the host constructor
			'the WinForms Application calls will give you grief, because a form will have been constructed already
			'this demonstrates, yet again, why global state is a bad thing
			Dim program As IAsynchronousHostableProgram = New FinalMixedDesignProgram()

			Return host.Run(program)
		End Function

		'//////////////////////////////////////////////////////////////////////

		''' <summary>
		''' Represents a program that can be run
		''' </summary>
		Public Interface IAsynchronousHostableProgram
			Event ExitRequested As EventHandler(Of EventArgs)
			Function StartAsync() As Task
		End Interface

		'//////////////////////////////////////////////////////////////////////

		''' <summary>
		''' Represents a host that uses Wpf to run the UI
		''' If you were so inclined, you could use the "WinFormsAddWpf" to create a "WinForms" host environment
		''' </summary>
		Public Class WpfHostEnvironment
			Private ReadOnly m_wpfApplication As System.Windows.Application

			Public Sub New()
				'the WPF application object will be the main UI loop
				m_wpfApplication = New System.Windows.Application With {.ShutdownMode = ShutdownMode.OnExplicitShutdown}
					'otherwise the application will close when all WPF windows are closed
				SynchronizationContext.SetSynchronizationContext(New DispatcherSynchronizationContext())

				'set the WinForms properties
				'notice how you don't need to do anything else with the Windows Forms Application (except handle exceptions)
				System.Windows.Forms.Application.EnableVisualStyles()
				System.Windows.Forms.Application.SetCompatibleTextRenderingDefault(False)

				'wire up exception handling
				AddHandler System.Windows.Forms.Application.ThreadException, AddressOf Application_ThreadException
				AddHandler m_wpfApplication.DispatcherUnhandledException, AddressOf m_wpfApplication_DispatcherUnhandledException
				AddHandler AppDomain.CurrentDomain.UnhandledException, AddressOf CurrentDomain_UnhandledException
				AddHandler TaskScheduler.UnobservedTaskException, AddressOf TaskScheduler_UnobservedTaskException
			End Sub

			Private Sub TaskScheduler_UnobservedTaskException(ByVal sender As Object, ByVal e As UnobservedTaskExceptionEventArgs)
				OnUnhandledException(New UnhandledExceptionEventArgs(e.Exception, False))
				e.SetObserved()
			End Sub
			Private Sub CurrentDomain_UnhandledException(ByVal sender As Object, ByVal e As UnhandledExceptionEventArgs)
				OnUnhandledException(e)
			End Sub
			Private Sub Application_ThreadException(ByVal sender As Object, ByVal e As ThreadExceptionEventArgs)
				OnUnhandledException(New UnhandledExceptionEventArgs(e.Exception, False))
			End Sub
			Private Sub m_wpfApplication_DispatcherUnhandledException(ByVal sender As Object, ByVal e As DispatcherUnhandledExceptionEventArgs)
				OnUnhandledException(New UnhandledExceptionEventArgs(e.Exception, False))
				e.Handled = True
			End Sub

			Public Event UnhandledException As EventHandler(Of UnhandledExceptionEventArgs)
			Protected Overridable Sub OnUnhandledException(ByVal args As UnhandledExceptionEventArgs)
				RaiseEvent UnhandledException(Me, args)
			End Sub

			''' <summary>
			''' Runs the application
			''' </summary>
			Public Function Run(ByVal programToHost As IAsynchronousHostableProgram) As Integer
				Try
					AddHandler programToHost.ExitRequested, AddressOf programToHost_ExitRequested

					Dim programStart As Task = programToHost.StartAsync()
					HandleExceptions(programStart)

					Return m_wpfApplication.Run()
				Finally
					'it is good practice to unsubscribe from events on objects that have longer lifetimes than the event handler
					RemoveHandler programToHost.ExitRequested, AddressOf programToHost_ExitRequested
				End Try
			End Function

			''' <summary>
			''' Runs the task and directs any unhandled exceptions into the global exception handler
			''' </summary>
			''' <param name="task"></param>
			Private async Sub HandleExceptions(ByVal task As Task)
				Try
					await Task.Yield() 'ensure this runs as a continuation
					Dim task As await
				Catch ex As Exception
					OnUnhandledException(New UnhandledExceptionEventArgs(ex, True))
				End Try
			End Sub

			Private Sub programToHost_ExitRequested(ByVal sender As Object, ByVal e As EventArgs)
				ShutDown()
			End Sub

			''' <summary>
			''' Shuts down the application
			''' </summary>
			Public Sub ShutDown()
				m_wpfApplication.Shutdown()
			End Sub
		End Class

		'//////////////////////////////////////////////////////////////////////


		''' <summary>
		''' Example program to run, same as WpfAddWinForms
		''' </summary>
		Public Class FinalMixedDesignProgram
			Implements IAsynchronousHostableProgram
			Private ReadOnly m_mainForm As Form1
			Public Sub New()
				m_mainForm = New Form1()
				AddHandler m_mainForm.FormClosed, AddressOf m_mainForm_FormClosed
			End Sub

			Public async Function StartAsync() As Task Implements IAsynchronousHostableProgram.StartAsync
				Dim splashScreen As New SplashScreenWpf() 'not disposable
				AddHandler splashScreen.Closed, AddressOf m_mainForm_FormClosed 'if user closes splash screen, let's quit
				splashScreen.Show()

				await m_mainForm.InitializeAsync()

				'http://blogs.msdn.com/b/wpfsdk/archive/2007/04/03/centering-wpf-windows-with-wpf-and-non-wpf-owner-windows.aspx
				Dim splashScreenHelper As New WindowInteropHelper(splashScreen)
				splashScreenHelper.Owner = m_mainForm.Handle

				m_mainForm.Show()
				RemoveHandler splashScreen.Closed, AddressOf m_mainForm_FormClosed
				splashScreen.Close()
			End Function

			Public Event ExitRequested As EventHandler(Of EventArgs) Implements IAsynchronousHostableProgram.ExitRequested
			Private Sub m_mainForm_FormClosed(ByVal sender As Object, ByVal e As EventArgs)
				OnExitRequested(EventArgs.Empty)
			End Sub

			Protected Overridable Sub OnExitRequested(ByVal e As EventArgs)
				RaiseEvent ExitRequested(Me, e)
			End Sub
		End Class
	End Class
End Namespace
