﻿Imports System.Threading
Imports System.Threading.Tasks
Imports System.Windows.Interop
Imports System.Windows.Threading
Imports System.Windows.Forms

Namespace SolSoft.ObjectOrientedGuiStartup.WindowsForms
	''' <summary>
	''' Using WinForms to run the app, but still supporting WPF.  
	''' This would seemingly be the scenario when adding WPF to a legacy application...however,
	''' Even if that's what you're doing, see WpfAddWinForms -- that way is more straightforward
	''' </summary>
	Friend Class WinFormsProgramAddWpf
		''' <summary>
		''' The main entry point for the application.
		''' </summary>
		<STAThread>
		Shared Sub Main()
			'The WinForms Application will be the main UI loop
			Application.EnableVisualStyles()
			Application.SetCompatibleTextRenderingDefault(False)
			SynchronizationContext.SetSynchronizationContext(New WindowsFormsSynchronizationContext())

			'it's more awkward to have WinForms run the show; in certain cirumstances, if you don't change the WPF Application shutdown mode,
			'the application will be destroyed when that window is closed (because of the default "OnLastWindowClose")
			'then when you open a second WPF window...boom
            Dim wpfApplication As System.Windows.Application = New System.Windows.Application With {.ShutdownMode = ShutdownMode.OnExplicitShutdown}
				'comment out this line to test, then click the button on the form to show another WPF window

			Dim p As New WinFormsProgramAddWpf()
			AddHandler p.ExitRequested, Sub(sender, e) Application.ExitThread()

			Dim programStart As Task = p.StartAsync()
			HandleExceptions(programStart)


			Application.Run()
			'now the wpf application can also shut down
			wpfApplication.Shutdown()
		End Sub

		Private Shared async Sub HandleExceptions(ByVal task As Task)
			Try
				await Task.Yield() 'ensure this runs as a continuation
                Await task
			Catch ex As Exception
				'deal with exception, either with message box
				'or delegating to general exception handling logic you may have wired up 
				'e.g. to Application.ThreadException and AppDomain.UnhandledException
				MessageBox.Show(ex.ToString())

				Application.ExitThread()
			End Try
		End Sub

		Private m_mainForm As Form1
		Private Sub New()
			m_mainForm = New Form1()
			AddHandler m_mainForm.FormClosed, AddressOf m_mainForm_FormClosed
		End Sub

		Public async Function StartAsync() As Task
			await Task.Yield() 'test that the synchronization context is properly set up

			Dim splashScreen As New SplashScreenWpf() 'not disposable, but I'm keeping the same structure
				AddHandler splashScreen.Closed, AddressOf m_mainForm_FormClosed 'if user closes splash screen, let's quit
				splashScreen.Show()

				await m_mainForm.InitializeAsync()

				'http://blogs.msdn.com/b/wpfsdk/archive/2007/04/03/centering-wpf-windows-with-wpf-and-non-wpf-owner-windows.aspx
				Dim splashScreenHelper As New WindowInteropHelper(splashScreen)
				splashScreenHelper.Owner = m_mainForm.Handle

				m_mainForm.Show()
				RemoveHandler splashScreen.Closed, AddressOf m_mainForm_FormClosed
				splashScreen.Close()


			'This lets you test
			Dim b As Button = New Button With {.Text = "Show WPF window", .AutoSize = True}
			m_mainForm.Controls.Add(b)
			AddHandler b.Click, Sub(sender, e) CType(New SplashScreenWpf(), SplashScreenWpf).Show()
		End Function

		Public Event ExitRequested As EventHandler(Of EventArgs)
		Private Sub m_mainForm_FormClosed(ByVal sender As Object, ByVal e As EventArgs)
			OnExitRequested(EventArgs.Empty)
		End Sub

		Protected Overridable Sub OnExitRequested(ByVal e As EventArgs)
			RaiseEvent ExitRequested(Me, e)
		End Sub
	End Class
End Namespace
