﻿Imports System.Threading
Imports System.Threading.Tasks
Imports System.Windows.Interop
Imports System.Windows.Threading

Namespace SolSoft.ObjectOrientedGuiStartup.WindowsForms
	''' <summary>
	''' Using WPF to run the app, but still supporting Windows Forms
	''' This way is more straightforward than the reverse, even if the application is primarily WinForms-based
	''' </summary>
	Friend Class WpfProgramAddWinForms
		''' <summary>
		''' The main entry point for the application.
		''' </summary>
		<STAThread>
		Shared Sub Main()
			'the WPF application object will be the main UI loop
            Dim wpfApplication As System.Windows.Application = New System.Windows.Application With {.ShutdownMode = ShutdownMode.OnExplicitShutdown}
				'otherwise the application will close when all WPF windows are closed
			SynchronizationContext.SetSynchronizationContext(New DispatcherSynchronizationContext())

			'set the WinForms properties
			'notice how you don't need to do anything else with the Windows Forms Application (except handle exceptions)
            System.Windows.Forms.Application.EnableVisualStyles()
            System.Windows.Forms.Application.SetCompatibleTextRenderingDefault(False)

			Dim p As New WpfProgramAddWinForms()
			AddHandler p.ExitRequested, Sub(sender, e) wpfApplication.Shutdown()

			Dim programStart As Task = p.StartAsync()
			HandleExceptions(programStart, wpfApplication)

			wpfApplication.Run()
		End Sub

        Private Shared Async Sub HandleExceptions(ByVal task As Task, ByVal wpfApplication As System.Windows.Application)
            Try
                Await task.Yield() 'ensure this runs as a continuation
                Await task
            Catch ex As Exception
                'deal with exception, either with message box
                'or delegating to general exception handling logic you may have wired up 
                'e.g. to Application.DispatcherUnhandledException and AppDomain.UnhandledException
                System.Windows.MessageBox.Show(ex.ToString())

                wpfApplication.Shutdown()
            End Try
        End Sub

		Private ReadOnly m_mainForm As Form1
		Private Sub New()
			m_mainForm = New Form1()
			AddHandler m_mainForm.FormClosed, AddressOf m_mainForm_FormClosed
		End Sub

		Public async Function StartAsync() As Task
			await Task.Yield() 'test that the synchronization context is properly set up

			Dim splashScreen As New SplashScreenWpf() 'not disposable, but I'm keeping the same structure
				AddHandler splashScreen.Closed, AddressOf m_mainForm_FormClosed 'if user closes splash screen, let's quit
				splashScreen.Show()

				await m_mainForm.InitializeAsync()

				'http://blogs.msdn.com/b/wpfsdk/archive/2007/04/03/centering-wpf-windows-with-wpf-and-non-wpf-owner-windows.aspx
				Dim splashScreenHelper As New WindowInteropHelper(splashScreen)
				splashScreenHelper.Owner = m_mainForm.Handle

				m_mainForm.Show()
				RemoveHandler splashScreen.Closed, AddressOf m_mainForm_FormClosed
				splashScreen.Close()
		End Function

		Public Event ExitRequested As EventHandler(Of EventArgs)
		Private Sub m_mainForm_FormClosed(ByVal sender As Object, ByVal e As EventArgs)
			OnExitRequested(EventArgs.Empty)
		End Sub

		Protected Overridable Sub OnExitRequested(ByVal e As EventArgs)
			RaiseEvent ExitRequested(Me, e)
		End Sub
	End Class
End Namespace
